/*=========================================================*/
/*                                                         */
/*         SOURCE FILE : DEMO_C.C                          */
/*         PACKAGE     : I2C_DEMO                          */
/*                                                         */
/*=========================================================*/

/*MPP:::xxxxxx=============================================*/
/*                                                         */
/* PACKAGE NAME: I2C_DEMO                                  */
/* DESCRIPTION:                                            */
/*    This demo program reads the time from a PCF8583 clock*/
/*    IC, and displays it to an LCD display (both available*/
/*    at the I2C demoboard.                                */
/*                                                         */
/*    Hours and minutes will be displayed on LCD display   */
/*    Dot between hours and minutes will blink             */
/*                                                         */
/*EMP======================================================*/


/*=========================================================*/
/*    I N C L U D E S                                      */
/*=========================================================*/
    #include "I2C_C.H"


/*=========================================================*/
/*    L O C A L   S Y M B O L   D E C L A R A T I O N S    */
/*=========================================================*/
    #define Clock_Adr       0xA2
    #define LCD_Adr         0x74
    #define Cl_Sub_Adr      0x01


/*=========================================================*/
/*    L O C A L   D A T A   D E F I N I T I O N S          */
/*=========================================================*/
    rom  char       LCD_Tab[]={0xFC,0x60,0xDA,0xF2,0x66,
                               0xB6,0x3E,0xE0,0xFE,0xE6};

 
/*=========================================================*/
/*    M A I N                                              */
/*=========================================================*/
void main()
{
    rom  char              *Tab_Ptr;
    data char              Time_Buffer[4];
    data char              *Time_Ptr;
    data char              LCD_Buffer[5];
    data char              *LCD_Ptr;
   
    I2C_INIT(0x22,&Time_Buffer,0);
    LCD_Buffer[0]=0; /* LCD control word*/
    Time_Buffer[0]=0;
    Time_Buffer[1]=0;
    I2C_WRITE(Clock_Adr,2,&Time_Buffer); /* Init clock*/

    while (1)      /* Program loop*/
    {
        I2C_READ_SUB(Clock_Adr,4,&Time_Buffer,Cl_Sub_Adr);
                                                /* Get time*/
        LCD_Ptr = &LCD_Buffer[1];   /* Init pointers*/
        Time_Ptr = &Time_Buffer[3];
        Tab_Ptr = (LCD_Tab+(*Time_Ptr >> 4));     /*10-HR's*/
        *(LCD_Ptr++) =*Tab_Ptr;
        Tab_Ptr = (LCD_Tab+(*(Time_Ptr--) & 0x0F)); /* HR's*/
        *(LCD_Ptr++) =*Tab_Ptr;
        Tab_Ptr = (LCD_Tab+(*Time_Ptr >> 4)); /* 10-MIN's*/
        *(LCD_Ptr++) = (*Tab_Ptr | 0x01); /* dp*/
        Tab_Ptr = (LCD_Tab+(*Time_Ptr & 0x0F)); /* MIN's*/
        *LCD_Ptr =*Tab_Ptr;
        Time_Ptr = &Time_Buffer[1]; /* Check sec's blinking*/
        LCD_Ptr = &LCD_Buffer[1];
        if ((*Time_Ptr & 0x01)>0)
            *LCD_Ptr = (*LCD_Ptr | 0x01);
        I2C_WRITE(LCD_Adr,5,&LCD_Buffer); /* Display time*/
    }
}
