$TITLE(I2C_Basic Functions)
;*==========================================================*
;*                                                          *
;*         SOURCE FILE : I2C_BASI.ASM                       *
;*         PACKAGE     : I2C                                *
;*                                                          *
;*==========================================================*
$DEBUG


;*==========================================================*
;*    I N C L U D E S                                       *
;*==========================================================*
$NOLIST
$INCLUDE(REG751.H)
$INCLUDE(I2C_DATA.GLO)
$INCLUDE(I2C_DATA.LOC)
$LIST


;*==========================================================*
;*    G L O B A L   R E F E R E N C E S                     *
;*==========================================================*
    EXTRN CODE(I2C_SLV_TRX)
    EXTRN CODE(I2C_SLV_RCV)
    EXTRN CODE(ADDR_RECOG)


;*==========================================================*
;*    G L O B A L   F U N C T I O N   D E F I N I T I O N S *
;*==========================================================*
    PUBLIC I2C_STRT_SLVAD
    PUBLIC I2C_RSTRT_SLVAD
    PUBLIC I2C_STOP
    PUBLIC I2C_TRX_BYTE
    PUBLIC I2C_TRX_ADDR
    PUBLIC I2C_RCV_BYTE
    PUBLIC I2C_RCV_ADDR
    PUBLIC I2C_TRX_BLOCK
    PUBLIC I2C_RCV_BLOCK
    PUBLIC ADDR_COMPARE


;*==========================================================*
;*    I N T E R R U P T   C O D E   S E G M :   T I M E R 1 *
;*==========================================================*
    CSEG AT 01BH


;*MPF:::I2C::I2C_INIT.ASM:I2C_TIME_OUT======================*
;*                                                          *
;* FUNCTION NAME:      I2C_TIME_OUT                         *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  I2C time out routine, clear timer interrupt, set the    *
;*  TIME_ERR bit. If the 8xC751 was IC bus master while the*
;*  interrupt occured (RECOVER = 1), an attempt to recover  *
;*  the bus is made.                                        *
;*  To recover, SDA and SCL are set, if SCL remains low, the*
;*  IC bus cannot ber recovered and the routine is left.   *
;*  If SCL is HIGH but SDA is low, 9 additional clocks are  *
;*  generated. If SDA becomes HIGH, a STOP is made          *
;*  If the 8xC751 was not IC bus master (RECOVER = 0), the *
;*  bus is released.                                        *
;*                                                          *
;*EMP=======================================================*
        SETB    CLRTI
        SETB    TIME_ERR
        CLR     TIRUN
        AJMP    TI_INT


;*==========================================================*
;*    C O D E   S E G M E N T                               *
;*==========================================================*
    I2C_DRIVER SEGMENT CODE
    RSEG I2C_DRIVER


;*MPP:::I2C::I2C_BASI=======================================*
;*                                                          *
;* SOURCE FILE:        I2C_BASI.ASM                         *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION: This file contains the basic I2C functions  *
;*   being: START, REP_START, STOP, TRX_SLV_ADDR, TRX_BYTE  *
;*   TRX_BLOCK, RCV BYTE, RCV_BLOCK.  The wait loops on ATN *
;*   are left if the ATN bit is set or if a TI interrupt    *
;*   occurs (time out). In case of a time out the program   *
;*   checks wether it is a "real" time out (max. time is    *
;*   exceeded). If yes, the program continues and will enter*
;*   a error routine, if no the ATN wait loop is reentered  *
;*                                                          *
;*EMP=======================================================*



;*MPF:::I2C::I2C_BASI.ASM:I2C_START=========================*
;*                                                          *
;* FUNCTION NAME:      I2C_START                            *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Generate a start condition on the IC bus, Set the      *
;*  MASTRQ bit. If 8xC751 has not become master on ATN,     *
;*  switch to receive mode and check if the own slave       *
;*  address is received.                                    *
;*                                                          *
;* INPUT: none                                              *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;* OUTPUT CONDITION: SCL is stretched                       *
;*                                                          *
;*EMP=======================================================*
I2C_STRT_SLVAD:
        SETB    TIRUN
        JB      STR,IS_MASTER       ;already started
        CLR     EI2                 ;disable I2C interrupt
        MOV     I2CFG,#I2C_START_CTRL
        ACALL   WAIT_ATN
IS_MASTER:
        JB      MASTER,I2C_TRX_ADDR
        MOV     I2CON,#C_STRT
        ACALL   I2C_RCV_ADDR
        JB      I2C_ERR,END_I2C_START
        ACALL   ADDR_COMPARE
START_ERR:
        SETB    I2C_ERR
END_I2C_START:
        RET


;*MPF:::I2C::I2C_BASI.ASM:I2C_REP_START=====================*
;*                                                          *
;* FUNCTION NAME:      I2C_REP_START                        *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Generate a repeated start condition on the IC bus      *
;*  The repeatead start is generated by setting the XSTR    *
;*  bit. If STR is not set (by hardware), the IC bus is    *
;*  released, no check for own slave address is done after a*
;*  repeated start.                                         *
;*                                                          *
;* INPUT: none                                              *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;* OUTPUT CONDITION: SCL is stretched                       *
;*                                                          *
;*EMP=======================================================*
I2C_RSTRT_SLVAD:
        MOV     I2CON,#S_RSTR
        ACALL   WAIT_ATN                ;wait for rising SCL
        JNB     DRDY,I2C_BASIC_ERR
        MOV     I2CON,#C_DRDY
        ACALL   WAIT_ATN                ;wait for rep start
        JNB     STR,I2C_BASIC_ERR
        SJMP    I2C_TRX_ADDR


;*MPF:::I2C::I2C_BASI.ASM:I2C_STOP==========================*
;*                                                          *
;* FUNCTION NAME:      I2C_STOP                             *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Generate a stop condition on the I2C bus                *
;*  The STOP conditionis generated by setting the XSTP bit. *
;*  If no error occurs, this function is left with I bus   *
;*  released and TI stopped. In case of an error the bus is *
;*  released in the message handler.                        *
;*                                                          *
;* INPUT: none                                              *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;*                                                          *
;*EMP=======================================================*
I2C_STOP:
        CLR     MASTRQ
        MOV     I2CON,#S_STP
        ACALL   WAIT_ATN                ;wait for rising SCL
        JNB     DRDY,I2C_BASIC_ERR
        MOV     I2CON,#C_DRDY
        ACALL   WAIT_ATN                ;wait for stop
        MOV     I2CON,#I2C_RELEASE
        CLR     TIRUN
        RET


;*MPF:::I2C::I2C_BASI.ASM:I2C_TRX_ADDR======================*
;*                                                          *
;* FUNCTION NAME:      I2C_TRX_ADDR                         *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  This function calls I2C_TRX_BYTE to transmit the        *
;*  slave address, if an arbitration is lost before the last*
;*  bit is transmitted, the function recieves the remaining *
;*  bits (receive mode), and checks wether the own slave    *
;*  address has been received (call ADDR_CMP).              *
;*                                                          *
;* INPUT: byte to transmit in ACC                           *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;* OUTPUT CONDITION: SCL is stretched                       *
;*                                                          *
;*EMP=======================================================*
I2C_TRX_ADDR:
        ACALL   I2C_TRX_BYTE
        JNB     I2C_ERR,END_TRX_ADDR
        DJNZ    BIT_CNT,CONTINUE
        AJMP    END_TRX_ADDR
CONTINUE:
        JNB     ARL,END_TRX_ADDR
        JB      STR,END_TRX_ADDR        ;parasitaire START
        JB      STP,END_TRX_ADDR        ;parasitaire STOP
        CLR     I2C_ERR                 ;clr err for slv func
        INC     BIT_CNT                 ;correct BIT_CNT
        CLR     0E0H                    ;RDAT = 0 to ACC.0
RCV_NEXT_BIT:
        MOV     I2CON,#C_XMTA+C_DRDY+C_ARL  ;rcv mode
        ACALL   WAIT_ATN
        JNB     DRDY,RESTORE_ERR
        MOV     C,RDAT
        RLC     A
        DJNZ    BIT_CNT,RCV_NEXT_BIT
        ACALL   ADDR_COMPARE
RESTORE_ERR:
        SETB    I2C_ERR                 ;set err for ret main
END_TRX_ADDR:
        RET



;*MPF:::I2C::I2C_BASI.ASM:I2C_ADDR_COMPARE==================*
;*                                                          *
;* FUNCTION NAME:      I2C_ADDR_COMPARE                     *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Compares the contents of the accumulator (received      *
;*  address) with the OWN_SLV_ADDR. If equal and the RWN    *
;*  bit is 0 (master transmit, slave receive) I2C_SLV_RCV is*
;*  called. If equal and the RWN bit is 1 (master receive,  *
;*  slave transmit) I2C_SLV_TRX is called. If not equal exit*
;*  I2C_ADDR_COMPARE                                        *
;*                                                          *
;* INPUT: received address in ACC                           *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;* OUTPUT CONDITION: SCL is stretched                       *
;*                                                          *
;*EMP=======================================================*
ADDR_COMPARE:
        XRL     A,OWN_SLV_ADDR
        JZ      SEND_ACK
        CJNE    A,#1,END_ADDR_COMPARE
SEND_ACK:
        MOV     I2DAT,#0
        ACALL   WAIT_ATN
        JNB     DRDY,END_ADDR_COMPARE
        JZ      SLAVE_RCV
        AJMP    I2C_SLV_TRX
SLAVE_RCV:
        AJMP    I2C_SLV_RCV
END_ADDR_COMPARE:
        RET



;*MPF:::I2C::I2C_BASI.ASM:I2C_BASIC_ERR=====================*
;*                                                          *
;* FUNCTION NAME:      I2C_BASIC_ERR                        *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Set the I2C_ERR bit. The message handler tests this bit *
;*                                                          *
;* INPUT: none                                              *
;* OUTPUT: I2C_ERROR = 1                                    *
;*                                                          *
;*EMP=======================================================*
I2C_BASIC_ERR:
        SETB    I2C_ERR
        RET


;*MPF:::I2C::I2C_BASI.ASM:I2C_TRX_BYTE======================*
;*                                                          *
;* FUNCTION NAME:      I2C_TRX_BYTE                         *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Transmit a byte over the I2C bus.                       *
;*  NOTE: The STR bit is cleared  here in stead of in the   *
;*        I2C_START routine, because there must be valid    *
;*        data in I2DAT before STR may be cleared (also     *
;*        releases the SCL line).                           *
;*                                                          *
;*  The I2C_TRX_BYTE function transmits a byte over the IC *
;*  bus, after the last bit has been transmitted,           *
;*  the function switches to receive mode to receive the    *
;*  acknowledge bit. If NACK is received, the NO_ACK bit is *
;*  set. If arbitration is lost or an error occurs during   *
;*  I2C_TRX_BYTE the function is exit with the I2C_ERR bit  *
;*  set.                                                    *
;*                                                          *
;* INPUT: byte to transmit in ACC                           *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;* OUTPUT CONDITION: SCL is stretched                       *
;*                                                          *
;*EMP=======================================================*
I2C_TRX_BYTE:
        MOV     BIT_CNT,#8
TRX_BIT:
        MOV     I2DAT,A             ;release SCL
        MOV     I2CON,#C_STRT       ;if STR clear STR
                                            ;else dummy MOV
        ACALL   WAIT_ATN
        JNB     DRDY,I2C_BASIC_ERR
        RL      A
        DJNZ    BIT_CNT,TRX_BIT
        MOV     I2CON,#C_XMTA+C_DRDY    ;receive mode
        ACALL   WAIT_ATN
        JNB     DRDY,I2C_BASIC_ERR
        JNB     RDAT,TRX_BYTE_RDY   ;stretch SCL
        SETB    NO_ACK
TRX_BYTE_RDY:
        RET


;*MPF:::I2C::I2C_BASI.ASM:I2C_RCV_BYTE======================*
;*                                                          *
;* FUNCTION NAME:      I2C_RCV_BYTE                         *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Receive a byte from the I2C bus                         *
;*  This is one function which receives a byte into acc.    *
;*  RCV_BYTE first releases the SCL and then receives the 8 *
;*  bits. If RCV_ADDR is called, the first bit is already in*
;*  the RDAT register, this must first be saved before the  *
;*  SCL is released.                                        *
;*                                                          *
;* INPUT: none                                              *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;*         if (! I2C_ERROR) received byte in ACC.           *
;*                                                          *
;*EMP=======================================================*
I2C_RCV_BYTE:
        MOV     I2CON,#C_XMTA+C_DRDY     ;rel. SCL, rcv mode
I2C_RCV_ADDR:
        MOV     BIT_CNT,#8
        CLR     A                   ;  rcv first bit
RCV_BIT:
        ACALL   WAIT_ATN
        JNB     DRDY,I2C_BASIC_ERR
        DJNZ    BIT_CNT,NOT_LAST_BIT
        MOV     C,RDAT
        RLC     A
        RET
NOT_LAST_BIT:
        ORL     A,I2DAT             ; save bit, rel. SCL
        RL      A
        SJMP    RCV_BIT


;*MPF:::I2C::I2C_BASI.ASM:I2C_TRX_BLOCK=====================*
;*                                                          *
;* FUNCTION NAME:      I2C_TRX_BLOCK                        *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Transmit a block of bytes over the I2C bus              *
;*  The I2C_TRX_BLOCK function transmits as  much bytes as  *
;*  defined in BUF_LEN (set R2), before the message handler *
;*  is called, BUF_LEN_1 or BUF_LEN_2 is copied into BUF_LEN*
;*                                                          *
;* INPUT: pointer to begin of block data in BUF_PTR (R0)    *
;*        byte counter in BUF_LEN (R2)                      *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;*                                                          *
;*EMP=======================================================*
I2C_TRX_BLOCK:
        MOV     A,@BUF_PTR          ;load byte
        ACALL   I2C_TRX_BYTE
        JB      I2C_ERR,END_TRX_BLOCK
        JB      NO_ACK,END_TRX_BLOCK
        INC     BUF_PTR
        DJNZ    BUF_LEN,I2C_TRX_BLOCK
END_TRX_BLOCK:
        RET


;*MPF:::I2C::I2C_BASI.ASM:I2C_RCV_BLOCK=====================*
;*                                                          *
;* FUNCTION NAME:      I2C_RCV_BLOCK                        *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*  Receive a block of bytes from the I2C bus               *
;*  The I2C_RCV_BLOCK function receives as much bytes as    *
;*  defined in BUF_LEN (set R2), before the message handler *
;*  is called, BUF_LEN_1 or BUF_LEN_2 is copied into BUF_LEN*
;*                                                          *
;* INPUT: pointer to begin of receive buffer BUF_PTR (R0)   *
;*        byte counter in BUF_LEN (R2)                      *
;* OUTPUT: I2C_ERROR (0, no error; 1, error)                *
;*         if (!I2C_ERROR)   received bytes in buffer.      *
;*                                                          *
;*EMP=======================================================*
ACK_RCV_BYTE:
        MOV     I2DAT,#0                ;send ACK
        ACALL   WAIT_ATN
        JNB     DRDY,I2C_BASIC_ERR
I2C_RCV_BLOCK:
        ACALL   I2C_RCV_BYTE
        JB      I2C_ERR,END_RCV_BLOCK
        MOV     @BUF_PTR,A              ;save byte
        INC     BUF_PTR
        DJNZ    BUF_LEN,ACK_RCV_BYTE
        MOV     I2DAT,#80H              ;send NACK
        ACALL   WAIT_ATN
        JNB     DRDY,I2C_BASIC_ERR
END_RCV_BLOCK:
        RET



;*MPF:::I2C::I2C_BASI.ASM:WAIT_ATN==========================*
;*                                                          *
;* FUNCTION NAME:      WAIT_ATN                             *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*   The WAIT_ATN function waits for the ATN bit to be set. *
;*   The function is left if the ATN bit is set or if the   *
;*   TIME_ERR bit is set. The TIME_ERR bit indicates that a *
;*   bus timeout has occurred. If the 8xC751 enters this    *
;*   function as a master, the RECOVER bit is set,          *
;*   indicating that in case of a timeout, a bus recover    *
;*   action must be started.                                *
;*                                                          *
;*EMP=======================================================*
WAIT_ATN:
        JNB     MASTER,WAIT
        SETB    RECOVER
WAIT:
        JB      TIME_ERR,END_WAIT
        JNB     ATN,WAIT
END_WAIT:
        CLR     RECOVER
        RET



;*MPF:::I2C::I2C_BASI.ASM:TI_INT============================*
;*                                                          *
;* FUNCTION NAME:      TI_INT                               *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*   The TI_INT handles the timeout interrupt. It is        *
;*   entered when a time out occcurs (during WAIT_ATN).     *
;*   The function is placed here to be sure that it is      *
;*   linked when placed in a library.                       *
;*                                                          *
;*EMP=======================================================*
TI_INT:
        CLR     092H
        ACALL   SCL_DELAY
        SETB    092H
        JNB     RECOVER,RET_INT
        SETB    SDA
        SETB    SCL
        JNB     SCL,RET_INT
        MOV     BUS_ERR_CLKS,#9           ;SCL = 1

RETRY_LOOP:
        CLR     SCL
        ACALL   SCL_DELAY
        SETB    SCL
        ACALL   SCL_DELAY
        JB      SDA,MAKE_STOP        ;if SDA = 1 make stop
        DJNZ    BUS_ERR_CLKS,RETRY_LOOP
        RETI
MAKE_STOP:
        CLR     SCL
        NOP
        CLR     SDA
        ACALL   SCL_DELAY
        SETB    SCL
        ACALL   SCL_DELAY
        SETB    SDA                  ;make stop condition
        ACALL   SCL_DELAY
        SETB    BUS_RECOVERED
RET_INT:
        RETI

SCL_DELAY:              ;delay of 9 periods (>= 6 micro sec.)
        NOP             ; ACALL(2) + 5 NOP (4) + RET (2)
        NOP
        NOP
        NOP
        NOP
        RET

;*==========================================================*
;*     H I S T O R Y                                        *
;*==========================================================*
;*                                                          *
;* 03-07-91    J.C. Pijnenburg  original version            *
;*                                                          *
;*==========================================================*
        END
