$TITLE(83C751 Multi Master I2C Routines)
$MOD751

;maski bitw rejestr I2CFG
BTIR			EQU	10h			;TIRUN
BMRQ			EQU	40h			;MASTRQ
;maski bitw rejestru I2CON 
BCXA			EQU	80h			;CXA
BIDLE			EQU	40h			;IDLE
BCDR			EQU	20h			;CDR
BCARL			EQU	10h			;CARL
BCSTR			EQU	08h			;CSTR
BCSTP			EQU	04h			;CSTP
BXSTR			EQU	02h			;XSTR
BXSTP			EQU	01h			;XSTP
;statusy przesyanych komunikatw:
SGO			EQU	10h			;pracujcy slave przetwarza komunikat
SRCVD			EQU	11h			;odebrano nowy komunikat w trybie slave
SRLNG			EQU	12h			;jako slave odebrano komunikat, zbyt
;dugi aby zmieci si w buforze
STXED			EQU	13h			;slave skompletowa transmisj 
SRERR			EQU	14h			;bd magistrali w trybie slave
MGO			EQU	20h			;pracujcy master przetwarza komunikat
MRCVED		EQU	21h			;master odebra komunikat od slave
MTXED			EQU	22h			;jako master, odebrano komunikat o 
;kompletnej transmisji danych
						;(slave potwierdzi wszystkie bajty)
MTXNAK		EQU	23h			;jako master, odcito komunikat, bo 
;slave nie potwierdzi odbioru bajtu
MTXNOSLV		EQU	24h			;jako master, nie odebrano potwierdzenia
;wysanego adresu slave
TIMOUT		EQU	30h			;przekroczenie limitu czasu operacji
NOTSTR		EQU	32h			;master nie rozpozna "I2C start"
;zmienne wykorzystywane w obsudze przerwania
MASCMD		DATA	20h
SUBADD		BIT	MASCMD.0
RPSTRT		BIT	MASCMD.1
SETMRQ		BIT	MASCMD.2

           DSEG    AT 24h
MSGSTAT:		DS		1			;status komunikacji I2C
MYADDR:    	DS		1			;wasny adres I2C
DESTADRW:  	DS		1			;adres przeznaczenia z R/W
DESSUBAD:  	DS		1			;sub-adres przeznaczenia
MASTCNT:   	DS		1			;liczba bajtw w komunikacie (master)
TITOCNT:   	DS		1			;licznik TIMEOUT
StackSave: 	DS		1			;adres stosu do odtworzenia
MasBuf:    	DS		4			;bufor transmisji master
SRcvBuf:   	DS		4			;bufor odbioru transmisji slave
STxBuf:    	DS		4			;bufor nadawania transmisji slave
RBufLen    	EQU	4			;dugo bufora w bajtach
;definicje portw uywanych przez aplikacj
TogLED     	BIT	P1.0		;dioda sygnalizacji statusu OK
ErrLED     	BIT	P1.1		;dioda sygnalizacji statusu BD
OnLED      	BIT	P1.3 		;dioda sygnalizacji zaczenia

;zmienne wykorzystywane przez aplikacj
APPFLAGS		DATA	21h   
TRQFLAG		BIT	APPFLAGS.0
SErrFLAG		BIT	APPFLAGS.1

FAILCNT:		DS		1
TOGCNT:		DS		1                               

;**************************************************************************
; Pocztek programu
;**************************************************************************           
CSEG

;tablica wektorw przerwa
ORG	0
			AJMP		Reset

ORG	1Bh								;Timer 1 (I2C timeout)
TimerI:
			SETB		CLRTI
			AJMP		TIISR
;przerwanie I2C
ORG	23h
I2CISR:
			CLR		EI2				;wyczenie przerwa I2C
			ACALL		XRETI 
			PUSH		PSW				;przechowanie rejestrw I PSW na stosie
			PUSH		ACC
			MOV		A,R0
			PUSH		ACC
			MOV		A,R1
			PUSH		ACC
			MOV		A,R2
			PUSH		ACC

			MOV		StackSave,SP	;przechowanie wskanika stosu
			CLR		TIRUN
			SETB		TIRUN

			JB			STP,NoGo
			JNB		MASTER,GoSlave	;praca w trybie slave
			MOV		MSGSTAT,#MGO
			JB			STR,GoMaster	;praca w trybie master
NoGo:
			MOV		MSGSTAT,#NOTSTR
			AJMP		Dismiss			;brak wanej sekwencji "I2C start"
XRETI:
			RETI

;**************************************************************************
; Gwne procedury obsugi transmisji danych
;************************************************************************** 

;SLAVE
GoSlave:
			MOV		MSGSTAT,#SGO
AddrRcv:
			ACALL		ClsRcv8
			JNB		DRDY,SmsgEnd	;odebrano bdny start lub stop przed
;skompletowaniem adresu
STstRW:
			MOV		C,ACC.0			;przechowaj bit R/W~ w CARRY
			CLR		ACC.0				;kasuj bit R/W, pozostaw adres
			JZ			GoIdl				;zignoruj, jeli to adres oglny
			JC			SlvTx				;w CARRY jest flaga R/W~,"1" to odczyt
											;slave powinien wysa komunikat


;zapis (slave powinien odebra komunikat)
;sprawdzenie, czy komuikat jest dla naszego adresu slave
SRcv2:
			CJNE		A,MYADDR,GoIdle;to nie nasz adres, zignoruj komunikat
			MOV		R1,#SrcvBuf		;ustaw bufor odbioru
			MOV		R2,#RbufLen+1
			SJMP		SRcv3

SRcvSto:
			MOV		@R1,A				;zapamitaj bajt
			Inc		R1					;nastpny adres w buforze
SRcv3:
			ACALL		AckRcv8
			JNB		DRDY,SrcvEnd	;wyjcie z ptli i koniec odbioru
			DJNZ		R2,SrcvSto		;zapamitaj bajt, jeli bufor nie 
											;jest przepeniony
											;zbyt duo bajtw - nie potwierdzaj odbioru
			MOV		MSGSTAT,#SRLNG	;inf.o odbiorze zbyt dugiego komunikatu
			ACALL		SLnRCvdR			;odbir nowych danych jako slave
			SJMP		GoIdle
;odebrano bajt ale brak sygnau DRDY, sprawd
SRcvEnd:
			CJNE		R0,#7,SrcvErr	;jeli nie zliczono 7 bitw, nie byo
;"I2C Start" lub "I2C Stop"
;odebrano kompletny komunikat
			MOV		MSGSTAT,#SRCVD      
;oblicz liczb odebranych bajtw
			MOV		A,R1                
			CLR		C
			SUBB		A,#SrcvBuf		;liczba bajtw w akumulatorze
			ACALL		SRCvdR			;obsu nowe dane jako slave
			SJMP		SMsgEnd
;to jest odczyt komunikatu, sprawd czy dla nas
SlvTx:
			NOP                      
STx2:
			CJNE		A,MYADDR,GoIdle	;tryb IDLE, gdy nie dla nas
			MOV		I2DAT,#0			;potwierdzenie odbioru adresu
			JNB		ATN,$				;czekaj na wysanie
			JNB		DRDY,SmsgEnd	;odebrano start lub stop przed
;przed odbiorem ack
			MOV		R1,#StxBuf		;ustawienie adresu bufora
STxlp:
			MOV		A,@R1				;pobranie bajtu z bufora
			INC		R1
			ACALL		XmByte
			JNB		DRDY,SmsgEnd	;byte wysyany nie by kompletny
			JNB		RDAT,STxlp		;potwierdzenie odbioru, transmisja
			MOV		I2CON,#BCDR+BIDLE	;nack od ukadu master
			MOV		MSGSTAT,#STXED
			ACALL		STXedR			;obsuga zdarzenia "slave przesa dane"
			AJMP		Dismiss

SRcvErr:
			MOV		MSGSTAT,#SRERR	;bity statusu bdw transmisji
			ACALL		SRErrR			;obsuga bdu w trybie slave
			SJMP		SMsgEnd
StxErr:
			MOV		MSGSTAT,#SRERR	;bity statusu bdw transmisji
			ACALL		SRErrR			;obsuga bdu w trybie slave
SMsgEnd:
			JB			MASTER,SMsgEnd2     
			JB			STR,GoSlave		;jeli odebrano start, przejd do trybu 
SMsgEnd2:
			AJMP		Dismiss			;slave
;koniec przetwarzania komunikatw w trybie slave
GoIdle:    
			AJMP		Dismiss


GoMaster:								;wysanie adresu I bitu R/W
			MOV		R1,#MasBuf		;bufor trybu master
			MOV		R2,MASTCNT		;liczba bajtw do transmisji
			MOV		A,DESTADRW		;adres slave z bitem R/W
			JB			SUBADD,GoMas2	;skok, jeli wymagany jest sub-adres
			ACALL		XmAddr
			JNB		DRDY,GM2
			JNB		ARL,GM3
GM2:
			AJMP		AdTxArl			;przegrana w arbitrau
GM3:
			JB			RDAT,Noslave	;brak ack dla wysanego adresu
			JB			ACC.0, MRcv		;sprawdzenie bitu R/W
			AJMP		MTx
;obsuga adresowania 10-bitowego (sub-adres)
GoMas2:	NOP				;potrzebny sub-adres
			CLR		ACC.0				;wylij bit zapisu z sub-adresem
			ACALL		XmAddr
			JNB		DRDY,GM4
			JNB		ARL,GM5
GM4:
			AJMP		AdTxArl			;przegrana w arbitrau podczas wysyania
;adresu
GM5:		JB			RDAT,Noslave	;brak ack po wysaniu adresu
			MOV		A,DESSUBAD
			ACALL		XmByte			;wysanie adresu
			JNB		DRDY,SMsgEnd2	;przegrana w arbitrau (gdy start/stop)
			JB			ARL,SMsgEnd2
			JB			RDAT,NoAck		;brak potwierdzenia odbioru sub-adresu
			MOV		A,DESTADRW		;zaaduj adres do akumulatora
			JNB		ACC.0,MTx		;zapis, wic wysyaj dane
;odczyt komuikatu wymaga powtrzonego warunku start I retransmisji
			MOV		I2CON,#BCDR+BXSTR	;wylij powtrzony start
			JNB		ATN,$
			MOV		I2CON,#BCDR		;kasowania flagi DRDY gdy przygotowanie
;do wysania powtrzonego warunku start
			JNB		ATN,$				;oczekiwanie na zmian ATN (wraz z STR)
			JNB		ARL,GM6
			AJMP		MarlEnd			;przegrana w arbitrau
GM6:
			ACALL		XmAddr			;powtrne przesanie adresu, tym razem z
;ustawionym bitem odczytu
			JNB		DRDY,GM7
			JNB		ARL,GM8
GM7:
			AJMP		AdTxArl			;przegrana w arbitrau
GM8:
			JB			RDAT,Noslave	;brak ack
			SJMP		MRcv				;odbir danych z ukadu slave

;zapis danych do slave, wysyanie dancych w trybie master
MTx:
			NOP
MTxLoop:
			MOV		A,@R1				;pobierz bajt z bufora
			INC		R1					;nastpny adres w buforze
			ACALL		XmByte
			JNB		DRDY,SMsgEnd2	;przegrana w arbitrau (gdy start/stop)
			JB			ARL,SMsgEnd2
			JB			RDAT,NoAck
			DJNZ		R2,MtxLoop		;ptla przesyania danych
			MOV		MSGSTAT,#MTXED	;informacja o skompletowaniu transmisji
			SJMP		MTxStop
NoSlave:
			MOV		MSGSTAT,#MTXNOSLV
			SJMP		MTxStop
NoAck:
			MOV		MSGSTAT,#MTXNAK
			SJMP		MTxStop

;ukad master odbiera dane
MRcv:
			ACALL		ClaRcv8			;odbir bajtu
			SJMP		MRcv2
MRcvLoop:	
			ACALL		AckRcv8
MRcv2:
			JNB		DRDY,Marl
			MOV		@R1,A					;zapamitaj bajt
			INC		R1						;nastpny adres
			DJNZ		R2,MRcvLoop
												;odebrano dan liczb bajtw, 
												;wylij nack
			MOV		I2DAT,#80h
      	JNB		ATN,$
			JNB		DRDY,MArl
			MOV		MSGSTAT,#MRCVED
;wylij stop lub powtrzony start
MTxStop:
			JNB		RPSTRT,MTxStop2	;sprawd, czy potrzebny powt. start
												;nie jeli wyzerowany RPSTRT.
			MOV		I2CON,#BCDR+BXSTR	;wylij powtrny start
			SJMP		MTxStop3
MTxStop2:
			MOV		C,SETMRQ				;danie dostpu w trybie master, jeli
			MOV		MASTRQ,C				;potrzebne przez ustawienie SETMRQ
			MOV		I2CON,#BCDR+BXSTP	;wylij sekwencj stop
MTxStop3:
			JNB		ATN,$					;oczekiwanie na wysanie
			MOV		I2CON,#BCDR			;kasowanie DRDY ustawionej podczas
												;transmisji stop lub powt. start
			JNB		ATN,$					;czekaj na ARL, STP or STR.
			JB			ARL,MarlEnd			;przegrana w arbitrau sprbuj wysa
												;stop lub ReStart.
;zakoczono transmisj w trybie master, mona przesa nastpny komunikat 
;lub opuci procedur
			ACALL		MastNext				;obsuga trybu master, moe przygotowa
												;do nastpnej transmisji komunikatu
			JNB		MASTRQ,MmsgEnd		;zakoczenie pracy, jeli stan MASTRQ
												;nie wskazuje, e master powinien
												;kontynuowa transmisj
			JNB		STR,MmsgEnd			;powrt z procedury obsugi przerwania
												;przed sygnaem start (jeli by stop
												;Timer 1 nie jest aktywny)
			AJMP		GoMaster				;ptla dla nastpnego komunikatu
MMsgEnd:										;koniec pracy w trybie master
			SJMP		Dismiss

;przerwanie pracy w trybie master na skutek przegranej w arbitrau
MArl:
			JNB		STR,MArl2			;jeli przegrana, pracuj jako slave
			AJMP		GoSlave
Marl2:
			AJMP		Dismiss
;przeczenie z trybi master do slave na skutek przegranej w arbitrau po
;zakoczeniu transmisji komuikatu;MASTRQ zosta skasowany,gdy prbowano 
;wys stop i musimy powtrnie fo ustawi w celu powtrzenia transmisji,gdy
;magistrala bdzie wolna
MArlEnd:
			SETB		MASTRQ				;zacz danie dostpu w trybie master
												;bdzie ono aktywne po wyjciu z trybu 
												;slave
			AJMP		MArl
;obsuga przegranej w arbitrau podczas transmisji adresu
AdTxArl:
			JB			STR,Marl				;nie-synchroniczny start or stop.
			JB			STP,MArl
;przeczenie z trybu master do trybu slave po przegranej w arbitrau
;podczas przesyania adresu - odbierz adres wysyany przez nowy ukad
;master
			CJNE		R0,#0,AdTxArl2		;R0 jest 0 przy wyjciu z XmAddr

			DEC		A						;wyliczenie adresu wygrywajcego
												;urzdzenia master przez odjcie 1 od
												;wysanego bajtu lsb adresu
			SJMP		AdAr3
AdTxArl2:
			RR			A						;odtwrz czciowo przesany akumulator
			MOV		R1,A					;i zapamitaj go w R1
			MOV		A,R0 
			MOV		DPTR,#MaskTable
			MOVC		A,@A+DPTR
			ANL		A,R1					;zerowanie bitu, przy transmisji ktrego
												;przegrana w arbitrau oraz poczenie 
												;go z wczeniej przesanymi - teraz
												;mona odebra dalsz cz adresu

			MOV		I2CON,#BCXA+BCARL	;zwolnienie linii zegara, zerowanie flag
			ACALL		RBit3					;odbir adresu
			JB			DRDY,AdAr3			;skocz, jeli odebrano poprawnie adres
			AJMP		SmsgEnd				;niespodziewany start/stop - koniec 
												;pracy w trybie slave
AdAr3:
			AJMP		STstRW				;wykonywane w celu sprawdzenia adresu

MaskTable:	DB	0ffh,7Eh,3Eh,1Eh,0Eh,06h,02h,00h
												;koniec procedury obsugi przerwania
Dismiss:
			ACALL		I2CDONE
			MOV		I2CON,#BCARL+BCSTP+BCDR+BCXA+BIDLE
			CLR		TIRUN
			POP		ACC
			MOV		R2,A
			POP		ACC
			MOV		R1,A
			POP		ACC
			MOV		R0,A
			POP		ACC
			POP		PSW
			SETB		EI2
			RETI								;powrt z procedury obsugi przerwania 

;**************************************************************************
; procedury wysania / odbioru danych
;************************************************************************** 
;XmAddr: przesanie adresu i bitu R/W  
;XmByte: przeanie bajtu
XmAddr:
			MOV		I2DAT,A				;przesanie pierwszego bitu, DRDY->0
			MOV		I2CON,#BCARL+BCSTR+BCSTP  
												;kasowanie statusu, zwolnienie SCL
			MOV		R0,#8					;R0 jako licznik bitw
			SJMP		XmBit2
XmByte:
			MOV		R0,#8
XmBit:
			MOV		I2DAT,A				;wylij bit
XmBit2:
			RL			A						;we nastpny bit
			JNB		ATN,$					;czekaj na wysanie bitu
			JNB		DRDY,XmBex			;testowanie bitu gotowoci
			DJNZ		R0,XmBit				;powtarzaj do wysania wszystkich bitw
			MOV		I2CON,#BCDR+BCXA	;przecz w tryb odbioru
			JNB		ATN,$					;czekaj na ack
XmBex:
			RET

;Odbir bajtu
;ClsRcv8		kasuje rejestr statusu (od warunku start) i odbiera bajt
;AckRcv8		wysya potwierdzenie a nastpnie odbiera nowy bajt
;				jeli start lub stop zostay odebrane bezporednio po ack 
;				AckRcv8 zwraca 9 w R0
;ClaRcv8		wycza stan "transmisja aktywna" i zwalnia magistral I2C
;				(po odbiorze ack)
;AKUMULATOR	zawiera odebrany bajt przy powrocie z procedury
;R0			uywany jest jako licznik bitw

ClsRcv8:
			MOV		I2CON,#BCARL+BCSTR+BCSTP+BCXA
;kasowanie rejestru statusu
			JNB		ATN,$
			JNB		DRDY,RCVex
			SJMP		Rcv8
AckRcv8:
			MOV		I2DAT,#0				;wysanie ack ("0")
			JNB		ATN,$
			JNB		DRDY,RCVerr			;bd magistrali - wyjcie
ClaRcv8:
			MOV		I2CON,#BCDR+BCXA	;kasuj bit statusu, zwolnij lini SCL
			JNB		ATN,$					;po wysaniu ack
Rcv8:
			MOV		R0,#7					;ustawienie licznika bitw 
			CLR		A						;inicjowanie odebranego bajtu
RBit:
			ORL		A,I2DAT				;odbierz bit, kasuj ATN
RBit2:
			RL			A						;przesunicie bitw w lewo
			JNB		ATN,$					;czekaj na nastpny bit
			JNB		DRDY,RCVex			;wyjd, jeli to nie by bit danych 
												;(mg to by start/stop, lub bd
												;transmisji)
RBit3:
			DJNZ		R0,Rbit				;powtrz dla 7 bitw
			MOV		C,RDAT				;pobierz ostatni bit, nie kasuj ATN
			RLC		A						;formuj odebrany bajt
RCVex:
			RET 

RCVerr:
			MOV		R0,#9					;bd, zwr niepoprawn liczb bajtw
			RET


;**************************************************************************
; procedura obsugi przerwania Timera 1 (timeout t.j. przekroczenie limitu
; czasu operacji)
;************************************************************************** 
TIISR:
			CLR		MASTRQ				;"rczny" reset
			MOV		I2CON,#BXSTP		; 
			MOV		I2CON,#BCXA+BCDR+BCARL+BCSTR+BCSTP
TI1:
			MOV		MSGSTAT,#TIMOUT	;status dla programu gwnego
TI2:
			MOV		A,#0FFh             
			CJNE		A,TITOCNT,TI3		;zwikszenie stanu licznika przerwa
			SJMP		TI4					;Timera 1
TI3:
			INC		TITOCNT
TI4:
			ACALL		RECOVER

			SETB		CLRTI					;kasowanie flagi przerwania Timera 1
			ACALL		XRETI					;kasowanie flagi trwajcego przerwania
			MOV		SP,StackSave		;odtworzenie wskanika stosu 
			AJMP		Dismiss				;wr do gwnej procedury I2C
        
;**************************************************************************
; prba odtworzenia dostpu do magistrali I2C
;************************************************************************** 
RECOVER:
			CLR		EA
			CLR		MASTRQ		;"rczny" reset
			MOV		I2CON,#BCXA+BIDLE+BCDR+BCARL+BCSTR+BCSTP
			CLR		SLAVEN				;Timer1 w trybie nie-I2C
			SETB		TIRUN					;uruchomienie Timera 1
			MOV		R1,#0ffh
DLY5:
			NOP
			NOP
			NOP
			DJNZ		R1,DLY5
			CLR		TIRUN
			SETB		CLRTI

			SETB		SCL        			;zwolnienie linii SCL i SDA
			SETB		SDA
			MOV		R1,#08h
RC7:
			CLR		SCL
			DB	0,0,0,0,0
			SETB		SCL
			DB	0,0,0,0,0
			DJNZ		R1,RC7
			CLR		SCL
			DB	0,0
			CLR		SDA
			DB	0,0
			SETB		SCL
			DB	0,0,0,0,0
			SETB		SDA
			DB	0,0,0,0,0					;"I2C stop"

Rex:
			MOV		I2CON,#BCXA+BCDR+BCARL+BCSTR+BCSTP
			SETB		EA			;zerowanie bitw wskanikw stanu
				;i zaczenie przerwa
			RET

;**************************************************************************
; przykadowy program gwny, "odbijanie komunikatw"
;**************************************************************************
;PING-PONG: zasady "gry"
;Kady komunikat przesany jest przez procesor w trybie master i zawiera
;jeden bajt danych. Procesor, ktry odbiera dane w trybie slave zwiksza
;bajt o jeden i wysya go w trybie master. Po przepenieniu (przekroczeniu
;wartoci 0xFF, bajt jest zerowany i "gra" trwa nadal. Diody LED su do
;sygnalizacji stanu (TogLED) gry. 
;
;nastawy pocztkowe (inicjujce):
;- ustawienie bitw CT1,CT0 rejestru I2CFG, odpowiednio do uytej
;- czstotliwoci zegara
;- ustawienie zmiennej MYADDR odpowiednio do podanego adresu I2C
;Wartoci te pobierane s z ROM, ze staych R_CTVAL i R_MYADDR

Reset:
			MOV		SP,#07h				;ustawienie stosu
			CLR		A
			MOV		DPTR,#R_CTVAL
			MOVC		A,@A+DPTR
			MOV		I2CFG,A				;ustawienie CT0 I CT1 
			CLR		A
			MOV		DPTR,#R_MYADDR
			MOVC		A,@A+DPTR			;ustawienie adresu MYADDR
			MOV		MYADDR,A		
			CLR		OnLED
Reset2:
			CLR		ErrLED				;migotanie diody ERRLED
			ACALL		LDELAY              
			SETB		ErrLED                                       
			CLR		SErrFLAG
			CLR		TRQFLAG
			MOV		FAILCNT,#50h
			SETB		TogLED
			MOV		TOGCNT,#050h		;inicjowanie licznika zmian stanu led

;Zaczenie trybu slave
			MOV		I2CON,#BIDLE		;interfejs czuwa do nastpnej sekw.start
			SETB		SLAVEN				;zezwolenie na prac w trybie slave

;zaczenie przerwa niezbdne zarwno w trybie master jak i slave
			SETB		ETI					;Timer 1
			SETB		EI2					;I2C
			SETB		EA						;globalne zezwolenie na przerwania

;ustalenie warunkw pracy w trybie master
			MOV		MASCMD,#0h 
			MOV		DPTR,#PongADDR
			CLR		A
			MOVC		A,@A+DPTR
			MOV		DESTADRW,A			;adres "partnera", lsb ma stan niski dla
												;transakcji typu "zapis"
			MOV		MASTCNT,#01h		;dugo komunikatu - 1 bajt

PPSTART:
			MOV		MasBuf,#00h 

;transmisja "PING"
PP2:
			SETB		TRQFLAG
			SETB		MASTRQ
			MOV		R1,#0ffh
PP22:
			JNB		TRQFLAG,PP3			;przesanie poprawne
			DJNZ		R1,PP22
MFAIL1:
			DJNZ		FAILCNT,PP2
			ACALL		RECOVER          
			SJMP		Reset2

;odbir "PONG"
PP3:
			MOV		R0,#0ffh				;programowy licznik timeout
PP31:
			MOV		R1,#0ffh
PP32:
			JB			TRQFLAG,PP2			;odebrano poprawnie, przejd do wysania
			JB			SErrFLAG,PP5
			DJNZ		R1,PP32
			DJNZ		R0,PP31
PPTO:
			ACALL		RECOVER				;timeout
			AJMP		Reset2
PP5:
			CLR		ErrLED				;bd odbioru
			ACALL		LDELAY
			SETB		ErrLED
			CLR		SErrFLAG
			AJMP		PPSTART
LDELAY:
			MOV		R2,#030h
LDELAY1:
			MOV		R1,#0ffh
			DJNZ		R1,$
			DJNZ		R2,LDELAY1
			RET
                              
;**************************************************************************
; procedury obsugi zdarze w trybach master i slave
;**************************************************************************
;Obsuga zdarze w trybie slave
;
;Procedury wywoywane przez obsug przerwania I2C, po skompletowaniu  
;transakcji przesyania komunikatu. Aplikacja po odebraniu komunikatu 
;wywouje procedur jego interpretacji SRCvdR. Wywoanie zachodzi w
;bezbdnej transmisji danych.
SRcvdR:
		 	NOP
			MOV		A,SRcvBuf
			JNZ		SR2             
			MOV		MasBuf,#01h			;odebrano warto inicjujc
			SJMP		SR3
SR2:
			INC		MasBuf				;przesunicie na nastpn pozycj bufora
			CJNE		A,MasBuf,ErrSR
			INC		MasBuf				;dane dla nastpnej transmisji odebrane
												;i zwikszone o 1
			DJNZ		TOGCNT,SR3			;stan diod LED zmienia si po penej
			CPL		TogLED				;transmisji ze wzgldu na szybko
			MOV		TOGCNT,#050h		;wizualizacji
SR3:
			CLR		SErrFLAG
			SETB		TRQFLAG
			RET
ErrSR:
			SETB		SErrFLAG
			RET

;SLnRcvdR	wywoywana, gdy odebrany w trybie slave komunikat jest zbyt
;				duy dla aktualnego rozmiaru bufora
;STXedR		wywoywana, gdy slave slave skompletowa transmisj w buforze
;				nie uywana w tej aplikacji
;SRErrR		obsuga bdu w trybie slave

SLnRcvdR: 
STXedR:   
SRErrR:
			JMP		ErrSR
                    
;MastNext	wywoywana gdy master skoczy transmisj lub zostaa ona
;				przerwana ze wzgldu na brak potwierdzenia przez slave
MastNext:
			MOV		A,MSGSTAT
			CJNE		A,#MTXED,MN1
			MOV		FAILCNT,#50h
			CLR		TRQFLAG
MN1:
			RET

;I2CDONE		wywoywana po zakoczeniu obsugi przerwania I2C. Jest to  
;				przykad monitorowania zdarze i przeprowadzenia operacji
;				odtworzenia magistrali I2C przy zakceniach transmisji
I2CDONE:
			MOV		A,MSGSTAT
			CJNE		A,#NOTSTR,I2CD1
			DJNZ		FAILCNT,I2CD1
			MOV		FAILCNT,#01h		;odebrano zbyt duo bdnych przerwa
			CLR		EI2					;od interfejsu I2C - wycz interfejs
I2CD1:
			RET

 ;parametry transmisji I2C: adres wasny i partnera, stan bitw preskalera
R_MYADDR:	DB		4Eh					;adres master
R_CTVAL:		DB		02h					;CT1 i CT0 - bity preskalera
PongADDR:	DB		4Ah					;adres partnera w grze

END
 
