;Software SPI Master
;wersja 1.0, z dnia 98.04.21, mikrokontroler AT90S1200

.include "1200def.inc"

;numery wersji programu
.equ	SW_MAJOR = 1
.equ	SW_MINOR = 0
.equ	HW_MAJOR = 0
.equ	HW_MINOR = 0

;definicje linii portw
.equ	sck	= 0			;PB0 pin
.equ	nss	= 1			;PB1 pin
.equ	mosi	= 2			;PB2 pin
.equ	miso	= 3			;PB3 pin

;definicje rejestrw
.def	spi_lo	=r0
.def	spi_hi	=r1
.def	temp		=r16


;defincje makr programu
;makra wymagaj zmiany jeli uywany jest inny port ni PB

.macro	ss_active		;wybranie ukadu slave
	cbi	portb,nss
.endm


.macro	ss_inactive		;wyczenie ukadu slave
	sbi	portb,nss
.endm


.macro	sck_hi			;ustawienie linii SCK w stanie wysokim
	sbi	portb,sck
.endm


.macro	sck_lo			;ustawienie linii SCK w stanie niskim
	cbi	portb,sck
.endm


.macro	mosi_hi			;ustawienie linii MOSI w stanie wysokim
	sbi	portb,mosi
.endm


.macro	mosi_lo			;ustawienie linii MOSI w stanie niskim
	cbi	portb,mosi
.endm


.macro	addi
	subi	@0, -@1		;odejmowanie wartoci przeciwnej do
.endm				;bezporedniej


.macro	set_delay		;ustawienie czasu opnienia
	subi	@0, (@1 << 5)
.endm

.macro	inc_delay
	subi	@0, -(1 << 5)	;warto liczby przesuni musi by identyczna
.endm				;jak wyej!!!


;program gwny
.cseg
.org	0
Rvect:	rjmp	Reset

;inicjalizacja linii programowego interfejsu SPI
;procedura musi by zmodyfikowana, jeli uyty bdzie inny port ni PB
;przy wywoaniu nie s wymagane adne parametry, procedura nie zwraca
;adnych wynikw swojej pracy ani te komunikatw o bdach 
init_spi:
	ss_inactive		;ustawienie stanu wysokiego na linii SS
	sbi	ddrb,nss	;SS jako wyjcie
	sck_lo			;ustaw lini SCK w stanie niskim
	sbi	ddrb,sck	;SCK jako wyjcie
	mosi_lo			;ustaw lini MOSI w stanie niskim
	sbi	ddrb,mosi	;MOSI jako wyjcie
	cbi	ddrb,miso	;bit ten jest zerowany po zaczeniu napicia
	ret			;zerowanie na wszelki wypadek

;procedura sprawdza stan linii SCK (powinna by w stanie niskim) przed
;wybraniem ukadu slave przy pomocy sygnau SS
ena_spi:
	sck_lo
	mosi_lo
	ss_active
	ret

;wyczenie ukadu slave poprze ustawienie linii SS
disa_spi:
	ss_inactive
	ret

;ta procedura wysya / odbiera 8 lub 16 bitowe sowo danych (w zalenoci
;od modyfikacji wykonanych przez uytkownika)
;linia SCK jest zerowana po zakoczeniu pracy procedury, linia SS pozostaje
;bez zmian  moliwe jest wic wielokrotne wywoywanie procedury w celu
;zapisu / odczytu kolejnych porcji bitw
rw_spi:	
	ldi	temp,16		;inicjowanie licznika bitw (tryb 16 bitw)
				;ustaw na 8 dla trybu 8-bitw
spi_loop:
	lsl	spi_lo		;dla trybu transmisji 8-bitw
	rol	spi_hi		;usu lini ROL SPI_HI

	brcc	lo_mosi
	mosi_hi
	rjmp	mosi_done	;czas na ustalenie linii MOSI
lo_mosi:
	mosi_lo
	nop			;rwnie i tu czas na ustalenie linii MOSI
mosi_done:
	sck_hi			;czas trwania stanu wysokiego impulsu SCK
set_delay temp,4		;(4 * 3) cykle delay; zakres od 1 do 7
time_hi:
	inc_delay temp
	brcs	time_hi
	sck_lo			;opnienie przed odczytem MISO
	set_delay temp,4
time_lo:
	inc_delay temp
	brcs	time_lo
	sbic	pinb,miso	;po opnieniu, czytaj MISO i zapamitaj w D0
	inc	spi_lo		;wczeniej D0=0, wic uywamy INC do
	dec	temp 		;ustawienia D0.
	brne	spi_loop
	ret

;*************************************************************
;program gwny, przykad aplikacji
;*************************************************************

Reset:				;procedura wykonywana po sygnale reset
	rcall	init_spi	;inicjowanie parametrw SPI
	ser	temp
	out	DDRD,temp
	rjmp	Main

Main:
ldi	R22,0xA3		;dane do przesania
	mov	spi_lo,R22
	mov	spi_hi,R22	;skasuj lini dla trybu 8-bitw
	rcall	ena_spi		;wybierz ukad slave
	rcall	rw_spi		;wylij i odbierz dane
	rcall	disa_spi	;wycz ukad slave
	rcall	use_spi_rcv	;procedura obsugi odebranych danych
	rjmp	Main

Use_spi_rcv:			;wyprowadzenie bajtu przez port PD
	out	PortD,R22
	ret

END
